﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
// Add a reference to C:\Program Files\Microsoft SQL Server\110\Shared\Microsoft.SqlServer.XEvent.Linq.dll for x64 architecture
// or C:\Program Files (x86)\Microsoft SQL Server\110\Tools\Binn\ManagementStudio\Microsoft.SqlServer.XEvent.Linq.dll for x86 architecture.
// and then use the namespace.
using Microsoft.SqlServer.XEvent.Linq;

namespace XeReader_BlogDemo_CTP3
{
    class Program
    {
        /// <summary>
        /// This function will iterate through every event in a set of files from a directory
        /// and return the event name followed by the list of fields and action and their
        /// values for the event.
        /// </summary>
        /// <param name="args"></param>                
        static void Main(string[] args)
        {
            // Create an instance of the enumerable object using a set of XEL files as the source.
            QueryableXEventData events = 
                new QueryableXEventData(@"C:\Temp\*.xel");

            /* Or read a list of XEL files as source
            string[] fileList = new string[2];
            fileList[0] = @"C:\Temp\demo_trace_0_129418467298110000.xel";
            fileList[1] = @"C:\Temp\demo_trace_0_129391767647570000.xel";
            QueryableXEventData events = new QueryableXEventData(fileList);
            */

            /* Specify XEL and XEM file
            string[] xelFiles = new string[1];
            xelFiles[0] = @"C:\Temp\demo_trace_0_129418467298110000.xel";
            string[] xemFiles = new string[1];
            xemFiles[1] = @"C:\Temp\demo_trace_0_129418467298110000.xem";
            QueryableXEventData events = new QueryableXEventData(xelFiles, xemFiles);
            */

            /* Use live feed 
            QueryableXEventData stream = new QueryableXEventData(
                    @"Data Source = (local); Initial Catalog = master; Integrated Security = SSPI", "MyRunningXEventSession",
                    EventStreamSourceOptions.EventStream, EventStreamCacheOptions.DoNotCache);
            */

            // Iterate through each event in the file.
            foreach (PublishedEvent evt in events)
            {
                // Optionally you can check for specific events rather than respond to each event.
                // Obviously this is going to run for everything. 
                if (evt.Name == evt.Name)
                {
                    // List the event name
                    Console.WriteLine(evt.Name);

                    // Iterate through each field in the current event and return its name and value.
                    foreach (PublishedEventField fld in evt.Fields)
                    {
                        Console.WriteLine("\tField: {0} = {1}", fld.Name, fld.Value);

                        // Optionally take an action if you find a specific field.
                        if (fld.Value.ToString() == "foo")
                        {
                            // Add really important code here.
                        }
                    }

                    // Iterate through each action in thecurrent event and return its name and value.
                    foreach (PublishedAction act in evt.Actions)
                    {
                        Console.WriteLine("\tAction: {0} = {1}", act.Name, act.Value);
                    }

                    /*
                     * If you want to evaluate using the friendly text that you would find  
                     * in dm_xe_map_values rather than the integer map_key. 
                     * You have to explicitly cast the field to a MapValue:
                     * 
                    if (((MapValue)evt.Fields["field_name"].Value).Value == "foo" )
                    */
                }

                
            }

            Console.Read();
        }
    }
}
